        ; A boot sector to power down VGA and HDA smoothly
        org 0x100               ; will be irrelevant

start:  call start2             ; find own offset
start2: pop bp                  ; get own offset + offset of start2
        sub bp,start2           ; BP now is a correction for offset 0

        xor ax,ax
        mov es,ax
        mov ax,0x1234
        mov [es:0x472],ax       ; mark for "warm" reboot
        mov ax,cs
        mov ds,ax
        mov es,ax

        mov bx,msg1             ; announce reboot
        call puts

        mov bl,'9'              ; 987654321...

kloop:  mov ax,0x0101
        int 0x16
        jz nokey
        xor ax,ax
        int 0x16
        cmp al,27
        jz reboot
nokey:  mov al,bl
        call putch
        dec bx
        cmp bl,'0'              ; did we already print the '0'?
        jb shutdown
        call wait1              ; else wait and keep counting down
        jmp short kloop


reboot: mov bx,msgboot
        call puts

        jmp word 0xffff:0x0000 ; reboot, simple...


shutdown:
        mov dx,0x1f6            ; primary IDE channel
        mov al,0xa0             ; master disk, not LBA
        out dx,al
        call gettime            ; just to waste some time
        mov al,0xe0             ; spin down command (0xe1 is spin up)
	inc dx			; command port
        out dx,al

        mov bx,msg2             ; announce spin down
        call puts

        mov bl,'5'              ; 54321...

dloop:  mov al,bl
        call putch
        dec bx
        cmp bl,'0'
        jb vgaoff
        call wait1
        jmp short dloop


vgaoff: mov bx,msg3
        call puts

        mov ax,0x1201           ; off (AL would be 00 for VGA on)
        mov bl,0x36             ; refresh on/off
        int 0x10                ; VGA

        mov dx,0x3c4            ; sequencer
        mov al,1                ; register 1
        out dx,al
        inc dx
        in al,dx
        and al,0xd0
        or al,0x20              ; set this bit to disable VGA...
        mov ah,al
        dec dx
        mov al,1                ; register 1
        out dx,al
        inc dx
        mov al,ah               ; new value
        out dx,al

        mov dx,0x3d4            ; simply assume color mode
        mov al,0x17             ; register 0x17
        mov ah,al
        out dx,al
        inc dx
        in al,dx
        and al,0x7f             ; stop CRTC
        xchg al,ah              ; remember new value in AH, get 0x17 in AL
        dec dx
        out dx,al
        mov al,ah               ; new value
        inc dx
        out dx,al

        sti
hloop:  hlt
        call gettime            ; just for fun
        jmp short hloop


gettime:                        ; fetch timer tick count
        push es
        mov ax,0x40
        mov es,ax
        mov ax,[es:0x6c]
        pop es
        ret                     ; return AX from timer


wait1:  push ax                 ; wait 1 second
        push bx
        call gettime
        add ax,18               ; 18.2 ticks are 1 second
        mov bx,ax
        sti
wloop:  hlt
        call gettime
        cmp ax,bx
        jnz wloop
        pop bx
        pop ax
        ret


putch:  push ax                 ; output char AL
        push bx
        mov ah,0x0e             ; TTY
        mov bx,7
        int 0x10                ; BIOS
        pop bx
        pop ax
        ret

puts:   push bx                 ; print string at BX, with BP correction
        add bx,bp
sloop:  mov al,[cs:bx]
        or al,al
        jz send
        inc bx
        call putch
        jmp short sloop
send:   pop bx
        ret


msg1    db "Preparing for shutdown... Press ESC to reboot now. ",0
msg2    db 13,10,"Waiting for first harddisk to spin down... ",0
msg3    db 13,10,"Ready. VGA off, CPU halted.",13,10,0

msgboot db 13,10,"Rebooting.",13,10,0

        times 0x1fc-$+$$ db 0   ; pad with zeroes
eof     db 0,0,0x55,0xaa        ; boot sector magic


